﻿
using System;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

/*       Transform a digital image into a vintage           */ 
/*         style grey scale image with oval form            */
/*                   www.mosismath.com                      */

namespace WindowsFormsApplication1
{
    public partial class MainWin : Form
    {
        double ellipseA = 500; 
        double ellipseB = 350;
        double band = 500; 
        Bitmap bmpSource;
        Bitmap bmpDest;
        String filePath = "";

        public MainWin()
        {
            InitializeComponent();
            pbSource.Height = 2112;
            pbSource.Width = 2816;
            pbDest.Height = 2112;
            pbDest.Width = 2816;

            bmpSource = new Bitmap(pbSource.InitialImage);
            bmpDest = new Bitmap(bmpSource.Width, bmpSource.Height);

            tbEllipseA.Text = ellipseA.ToString();
            tbEllipseB.Text = ellipseB.ToString();
            tbBand.Text = band.ToString();
        }


        private void button1_Click(object sender, EventArgs e)
        {
            int x, y;
            double R, G, B;
            double centerX, centerY;
  
            centerX = bmpSource.Width / 2;
            centerY = bmpSource.Height / 2;

            for (x = 1; x < bmpSource.Width - 1; x++)
            {
                for (y = 1; y < bmpSource.Height - 1; y++)
                {
                    Color pix1 = bmpSource.GetPixel(x, y);
                    double grayscale = (pix1.R + pix1.G + pix1.B) / 3.0;
                    double f = ellipseB / ellipseA;
                    double tempA = Math.Sqrt(((x - centerX)* (x - centerX)*f*f + (y - centerY)* (y - centerY))/f/f);

                    // compute the fading and gray scale
                    if (tempA > ellipseA)
                    {

                        double fader = tempA - ellipseA;
                        if (fader > band)
                            fader = band;
                        if (fader < 0)
                            fader = 0;
                        fader = fader / band;
                         R = grayscale * (1.0 - fader) + 255.0 * fader;
                         G = grayscale * (1.0 - fader) + 255.0 * fader;
                         B = grayscale * (1.0 - fader) + 255.0 * fader;
                    }
                    else
                    {
                        R = grayscale;
                        G = grayscale;
                        B = grayscale;
                    }

                    if (R < 0)
                        R = 0;
                    else
                    {
                        if (R > 255)
                            R = 255;
                    }
                    if (G < 0)
                        G = 0;
                    else
                    {
                        if (G > 255)
                            G = 255;
                    }
                    if (B < 0)
                        B = 0;
                    else
                    {
                        if (B > 255)
                            B = 255;
                    }

                    // set the corresponding pixel in the destination bitmap
                    Color nPix = Color.FromArgb((int)(R), (int)(G), (int)(B));
                    bmpDest.SetPixel(x, y, nPix);
                }
            }
                
            pbDest.Image = bmpDest;
            if(filePath != "")
                pbDest.Image.Save(filePath + "\\Output.jpg");
            else
                pbDest.Image.Save(Application.StartupPath + "\\Output.jpg");
            MessageBox.Show("Done :-)");
        }

        private void btOpenFile_Click(object sender, EventArgs e)
        {
            MemoryStream imageStream;
            byte[] buffer;

            if (openFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                filePath = Path.GetDirectoryName(openFileDialog.FileName);
                if (openFileDialog.FileName != null)
                {
                    FileStream fileStream1 = new FileStream(openFileDialog.FileName, FileMode.Open, FileAccess.Read);
                    try
                    {
                        int length = (int)fileStream1.Length;
                        buffer = new byte[length];
                        int count;
                        int sum = 0;

                        while ((count = fileStream1.Read(buffer, sum, length - sum)) > 0)
                            sum += count;
                    }
                    finally
                    {
                        fileStream1.Close();
                    }
                    imageStream = new MemoryStream(buffer, 0, buffer.Length);
                    string tempString1 = System.Text.Encoding.ASCII.GetString(buffer);
                    char[] bufferJpeg1 = new char[tempString1.Length];
                    tempString1.CopyTo(0, bufferJpeg1, 0, tempString1.Length);
                    imageStream.Position = 0;

                    // The source bitmap
                    bmpSource = new Bitmap(imageStream);

                    // The target bitmap
                    bmpDest = new Bitmap(bmpSource.Width, bmpSource.Height);

                    // Set the size of the picture boxes
                    pbSource.Image = bmpSource;
                    pbSource.Height = bmpSource.Height;
                    pbSource.Width = bmpSource.Width;
                    pbDest.Image = bmpSource;
                    pbDest.Height = bmpSource.Height;
                    pbDest.Width = bmpSource.Width;
                }
            }
        }

        private void tbEllipseA_Leave(object sender, EventArgs e)
        {
            try
            {
                ellipseA = Convert.ToInt32(tbEllipseA.Text);
                if(ellipseA < 0)
                {
                    tbEllipseA.Text = "500";
                    ellipseA = 500;
                }
            }
            catch
            {
                tbEllipseA.Text = "500";
                ellipseA = 500;
            }
        }

        private void tbEllipseB_Leave(object sender, EventArgs e)
        {
            try
            {
                ellipseB = Convert.ToInt32(tbEllipseB.Text);
                if (ellipseB < 0)
                {
                    tbEllipseB.Text = "300";
                    ellipseB = 300;
                }
            }
            catch
            {
                tbEllipseB.Text = "300";
                ellipseB = 300;
            }
        }

        private void tbBand_Leave(object sender, EventArgs e)
        {
            try
            {
                band = Convert.ToInt32(tbBand.Text);
                if (ellipseB < 0)
                {
                    tbBand.Text = "300";
                    band = 300;
                }
            }
            catch
            {
                tbBand.Text = "300";
                band = 300;
            }
        }
    }
}
